function [X B E] = SVDfilter(x,n)
% X = SVDfilter(x,n);
%[X B E] = SVDfilter(x,n);
%
%SVDfilter filters the data x: [observations time], using the n most 
%relevant dimensions.
%
%Variables:
%   x       =   The input data: [observations time].
%   n       =   The number of dimensions retained in the filtered data.
%   X       =   The filtered signal: [observations time].
%   B       =   The basis used.
%   E       =  *The energy distribution of the components.


if(length(n) == 1)
    d = 1:n;
end
[T E B] = svd(x');
E = diag(E)';
E = E./sum(E);

[Nx Nt] = size(x);
X = zeros(Nx,Nt,Nx);

Xp = getProjs(x,B);
for k = d
    X(:,:,d(k)) = getInvProjs(Xp,B,d(k));
end
X = squeeze(sum(X,3));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function projs = getProjs(data,Eout);
%projs = getProjs(data,Eout);

if(numdim(data) == 2)
    projs = Eout'*data;
else
    len = size(data,3); Eoutt = Eout';
    projs = zeros([size(Eout,2) size(data,2) size(data,3)]);
    for i = 1:len
        projs(:,:,i) = Eoutt*data(:,:,i);
    end
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function data = getInvProjs(projs,Eout,ind);
%projs = getInvProjs(projs,Eout,Indices);

if(nargin < 3)
    ind = 1:size(projs,1);
end
if(numdim(projs) < 3)
    ind = setdiff(1:size(projs,1),ind);
    projs(ind,:) = 0;
    data = Eout*projs;
else
    len = size(projs,3);
    data = zeros(size(projs));
    for i = 1:len
        data(:,:,i) = getInvProjs(projs(:,:,i),Eout,ind);
    end
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%